/*
    This file is part of GnuCode.
    Copyright (C) 2012 Jacob Dawid, jacob.dawid@googlemail.com

    GnuCode is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    GnuCode is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GnuCode. If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtGui>
#include <QIcon>
#include "finddialog.h"

find_dialog::find_dialog (QsciScintilla* edit_area, QWidget *p)
  : QDialog (p)
{
  setWindowTitle ("Find and Replace");
  setWindowIcon (QIcon(":/actions/icons/logo.png"));

  _search_label = new QLabel (tr ("Find &what:"));
  _search_line_edit = new QLineEdit;
  _search_label->setBuddy (_search_line_edit);
  _replace_label = new QLabel (tr ("Re&place with:"));
  _replace_line_edit = new QLineEdit;
  _replace_label->setBuddy (_replace_line_edit);

  _case_check_box = new QCheckBox (tr ("Match &case"));
  _from_start_check_box = new QCheckBox (tr ("Search from &start"));
  _wrap_check_box = new QCheckBox (tr ("&Wrap while searching"));
  _wrap_check_box->setChecked(true);
  _find_next_button = new QPushButton (tr ("&Find Next"));
  _find_next_button->setDefault (true);
  _replace_button = new QPushButton (tr ("&Replace"));
  _replace_all_button = new QPushButton (tr ("Replace &All"));

  _more_button = new QPushButton (tr ("&More"));
  _more_button->setCheckable (true);
  _more_button->setAutoDefault (false);

  _button_box = new QDialogButtonBox (Qt::Vertical);
  _button_box->addButton (_find_next_button, QDialogButtonBox::ActionRole);
  _button_box->addButton (_replace_button, QDialogButtonBox::ActionRole);
  _button_box->addButton (_replace_all_button, QDialogButtonBox::ActionRole);
  _button_box->addButton (_more_button, QDialogButtonBox::ActionRole);

  _extension = new QWidget (this);
  _whole_words_check_box = new QCheckBox (tr ("&Whole words"));
  _regex_check_box = new QCheckBox (tr ("Regular E&xpressions"));
  _backward_check_box = new QCheckBox (tr ("Search &backward"));
  _search_selection_check_box = new QCheckBox (tr ("Search se&lection"));
  _search_selection_check_box->setCheckable (false); // TODO: Not implemented.
  _search_selection_check_box->setEnabled (false);

  _edit_area = edit_area;
  connect (_find_next_button,   SIGNAL (clicked ()),
           this,                SLOT (search_next ()));
  connect (_more_button,        SIGNAL (toggled (bool)),
           _extension,          SLOT (setVisible (bool)));
  connect (_replace_button,     SIGNAL (clicked ()),
           this,                SLOT (replace ()));
  connect (_replace_all_button, SIGNAL (clicked ()),
           this,                SLOT (replace_all ()));

  QVBoxLayout *extension_layout = new QVBoxLayout ();
  extension_layout->setMargin (0);
  extension_layout->addWidget (_whole_words_check_box);
  extension_layout->addWidget (_backward_check_box);
  extension_layout->addWidget (_search_selection_check_box);
  _extension->setLayout (extension_layout);

  QGridLayout *top_left_layout = new QGridLayout;
  top_left_layout->addWidget (_search_label, 1, 1);
  top_left_layout->addWidget (_search_line_edit, 1, 2);
  top_left_layout->addWidget (_replace_label, 2, 1);
  top_left_layout->addWidget (_replace_line_edit, 2, 2);

  QVBoxLayout *left_layout = new QVBoxLayout;
  left_layout->addLayout (top_left_layout);
  left_layout->insertStretch (1, 5);
  left_layout->addWidget (_case_check_box);
  left_layout->addWidget (_from_start_check_box);
  left_layout->addWidget (_wrap_check_box);
  left_layout->addWidget (_regex_check_box);

  QGridLayout *main_layout = new QGridLayout;
  main_layout->setSizeConstraint (QLayout::SetFixedSize);
  main_layout->addLayout (left_layout, 0, 0);
  main_layout->addWidget (_button_box, 0, 1);
  main_layout->addWidget (_extension, 1, 0, 1, 2);
  setLayout (main_layout);

  _extension->hide ();
}

void
find_dialog::search_next ()
{
  int line = -1, col = -1;

  _find_result_available = false;
  if (_from_start_check_box->isChecked ())
    {
      line = 1;
      col  = 1;
    }

  if (_edit_area)
    {
      _find_result_available = _edit_area->findFirst (_search_line_edit->text (),
                                                      _regex_check_box->isChecked (),
                                                      _case_check_box->isChecked (),
                                                      _whole_words_check_box->isChecked (),
                                                      _wrap_check_box->isChecked (),
                                                      !_backward_check_box->isChecked (),
                                                      line,col,
                                                      true
#ifdef HAVE_FINDFIRST_MODERN
                                                      , true
#endif
                                                      );
    }
}


void
find_dialog::replace ()
{
  if (_edit_area)
    {
      _edit_area->replace (_replace_line_edit->text ());
      _edit_area->findNext();
    }
}

void
find_dialog::replace_all ()
{
  int count = 0;

  // check whether find & replace srings are different (avoid endless loop!)
  int strDiff;
  Qt::CaseSensitivity cs;
  if (_case_check_box->isChecked())
    {
      cs = Qt::CaseSensitive;
    }
  else
    {
      cs = Qt::CaseInsensitive;
    }
  strDiff = QString::compare(_search_line_edit->text(),_replace_line_edit->text(),cs);

  // replace all if strings are different
  if (_edit_area && strDiff )
    {
      search_next ();  // find first occurence
      while (_find_result_available)   // while search string is found
        {
          _edit_area->replace (_replace_line_edit->text ());   // replace
          count++;                                             // inc counter
          _find_result_available = _edit_area->findNext();                     // and find next
        }
    }
  // TODO: Show number of replaced strings
}
