/*
    This file is part of GnuCode.
    Copyright (C) 2012 Jacob Dawid, jacob.dawid@googlemail.com

    GnuCode is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    GnuCode is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GnuCode. If not, see <http://www.gnu.org/licenses/>.
*/

#include "mainwindow.h"
#include "ui-mainwindow.h"
#include "sourcecodeeditor.h"

#include <QFileDialog>
#include <QMenuBar>
#include <QDebug>
#include <QMdiSubWindow>
#include <QFileInfo>
#include <QProcess>

MainWindow::MainWindow (QWidget * parent):
QMainWindow (parent), ui (new Ui::MainWindow)
{
  ui->setupUi (this);

  // Project file system tree
  ui->fileSystemTreeView->setModel (&_file_system_model);
  connect (ui->fileSystemTreeView, SIGNAL (activated (QModelIndex)), this,
	   SLOT (file_activated (QModelIndex)));

  // Project handling
  connect (ui->actionOpen, SIGNAL (triggered ()), this,
	   SLOT (open_project ()));
  connect (ui->actionNew, SIGNAL (triggered ()), this,
           SLOT (new_project ()));
  connect (ui->actionClose, SIGNAL (triggered ()), this,
           SLOT (close_project ()));
  
  // Build steps
  connect (ui->actionBootstrap, SIGNAL (triggered ()), this,
	   SLOT (run_bootstrap ()));
  connect (ui->actionConfigure, SIGNAL (triggered ()), this,
	   SLOT (run_configure ()));
  connect (ui->actionMake, SIGNAL (triggered ()), this,
           SLOT (run_make ()));

  // Deployment steps
  connect (ui->actionInstall, SIGNAL (triggered ()), this,
	   SLOT (run_install ()));
  connect (ui->actionTarball, SIGNAL (triggered ()), this,
           SLOT (run_tarball ()));
  connect (ui->actionPackage, SIGNAL (triggered ()), this,
           SLOT (run_package ()));

  // Terminal handling
  connect (&_process, SIGNAL (readyReadStandardOutput ()), this,
	   SLOT (process_write_output ()));
  connect (&_process, SIGNAL (readyReadStandardError ()), this,
	   SLOT (process_write_error ()));
  connect (&_process, SIGNAL (finished (int)), this,
	   SLOT (process_finished (int)));
  connect (&_process, SIGNAL (error (QProcess::ProcessError)), this,
	   SLOT (process_error (QProcess::ProcessError)));
  connect (ui->commandLineEdit, SIGNAL (returnPressed ()), this,
	   SLOT (command_return_pressed ()));
           
  showMaximized ();
}

MainWindow::~MainWindow ()
{
  delete ui;
}

void
MainWindow::new_project ()
{
  // TODO: Implement.
  ui->outputTextEdit->append("This function has not been implemented.");
}

void
MainWindow::open_project ()
{
  QString file_name =
    QFileDialog::getOpenFileName (this, "Open Project", QString (),
				  "Makefile.am");
  if (!file_name.isEmpty ())
    {
      _current_project_path = QFileInfo (file_name).absoluteDir ();
      _file_system_model.setRootPath (_current_project_path.absolutePath ());
      ui->fileSystemTreeView->setRootIndex (_file_system_model.
					    index (_current_project_path.
						   absolutePath ()));
      ui->fileSystemTreeView->header ()->
	setResizeMode (QHeaderView::ResizeToContents);
      QDir::setCurrent (_current_project_path.absolutePath ());
    }
}

void
MainWindow::close_project ()
{
  // TODO: Implement.
  ui->outputTextEdit->append("This function has not been implemented.");
}

void
MainWindow::file_activated (QModelIndex index)
{
  QString file_name = _file_system_model.filePath (index);
  QFileInfo file_info (file_name);
  if (!file_info.isDir ())
    {
      SourceCodeEditor *editor = new SourceCodeEditor (this);
      editor->setFileName (file_name);
      QMdiSubWindow *subWindow = ui->mdiArea->addSubWindow (editor);
      subWindow->show ();
    }
}

void
MainWindow::run_bootstrap ()
{
  ui->outputTextEdit->append ("Running bootstrap..");
  _process.start ("./bootstrap");
}

void
MainWindow::run_configure ()
{
  ui->outputTextEdit->append ("Running configure..");
  _process.start ("./configure");
}

void
MainWindow::run_make ()
{
  ui->outputTextEdit->append ("Running make..");
  _process.start ("make all");
}

void
MainWindow::run_install ()
{
  ui->outputTextEdit->append ("Running install..");
  _process.start ("gksudo make install");
}

void
MainWindow::run_tarball ()
{
  // TODO: Implement.
  ui->outputTextEdit->append("This function has not been implemented.");
}

void
MainWindow::run_package ()
{
  // TODO: Implement.
  ui->outputTextEdit->append("This function has not been implemented.");
}

void
MainWindow::process_write_output ()
{
  ui->outputTextEdit->append ("<font color=\"#FFFFFF\">" +
			      QString::fromLocal8Bit (_process.
						      readAllStandardOutput
						      ()) + "</font>");
}

void
MainWindow::process_write_error ()
{
  ui->outputTextEdit->append ("<font color=\"#FF0000\">" +
			      QString::fromLocal8Bit (_process.
						      readAllStandardError ())
			      + "</font>");
}

void
MainWindow::process_finished (int return_code)
{
  ui->outputTextEdit->
    append (QString
	    ("<font color=\"%1\">Process finished with code %2.</font>").
	    arg (return_code == 0 ? "#00FF00" : "#FF0000").arg (return_code));
}

void
MainWindow::process_error (QProcess::ProcessError error)
{
  Q_UNUSED (error);
  ui->outputTextEdit->append (_process.errorString ());
}

void
MainWindow::command_return_pressed ()
{
  QString command = ui->commandLineEdit->text ();
  ui->commandLineEdit->clear ();
  ui->outputTextEdit->append (QString ("$ %1").arg (command));
  _process.start (command);
}
