/*
    This file is part of GnuCode.
    Copyright (C) 2012 Jacob Dawid, jacob.dawid@googlemail.com

    GnuCode is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    GnuCode is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GnuCode. If not, see <http://www.gnu.org/licenses/>.
*/

#include "sourcecodeeditor.h"
#include "ui-sourcecodeeditor.h"
#include <QVBoxLayout>
#include <QFile>
#include <QFileInfo>
#include <QDebug>

SourceCodeEditor::SourceCodeEditor (QWidget * parent):
QWidget (parent), ui (new Ui::SourceCodeEditor)
{
  ui->setupUi (this);

  _menuBar = new QMenuBar (this);
  QMenu *fileMenu = _menuBar->addMenu ("File");
  QAction *saveAction = fileMenu->addAction ("Save");
  saveAction->setShortcut (Qt::CTRL + Qt::Key_S);

  QAction *searchAction = fileMenu->addAction ("Search");
  searchAction->setShortcut (Qt::CTRL + Qt::Key_F);
  
  QFont font = QFont ("Monospace", 10);
  QFont boldFont = font;
  boldFont.setBold (true);

  _editor = new QsciScintilla (this);
  QsciLexerCPP *cppLexer = new QsciLexerCPP ();
  cppLexer->setFont (font);
  cppLexer->setColor (QColor (100, 255, 100), QsciLexerCPP::Comment);
  cppLexer->setColor (QColor (100, 255, 100), QsciLexerCPP::CommentLine);
  cppLexer->setColor (QColor (105, 105, 255), QsciLexerCPP::CommentDoc);
  cppLexer->setColor (QColor (105, 105, 255), QsciLexerCPP::CommentLineDoc);
  
  cppLexer->setColor (QColor (100, 155, 255), QsciLexerCPP::Keyword);
  cppLexer->setFont (boldFont, QsciLexerCPP::Keyword);
  
  cppLexer->setColor (QColor (255, 155, 255), QsciLexerCPP::PreProcessor);
  cppLexer->setFont (boldFont, QsciLexerCPP::PreProcessor);
  
  cppLexer->setColor (QColor (255, 255, 255), QsciLexerCPP::Default);
  cppLexer->setColor (QColor (255, 255, 255), QsciLexerCPP::Identifier);
  cppLexer->setColor (QColor (100, 255, 255), QsciLexerCPP::Operator);
  
  
  cppLexer->setColor (QColor (200, 255, 255), QsciLexerCPP::Number);
  cppLexer->setColor (QColor (200, 255, 255), QsciLexerCPP::SingleQuotedString);
  cppLexer->setColor (QColor (200, 255, 255), QsciLexerCPP::DoubleQuotedString);
  cppLexer->setColor (QColor (200, 255, 255), QsciLexerCPP::RawString);
  
  cppLexer->setDefaultColor (QColor (225, 225, 225));
  cppLexer->setDefaultPaper (QColor (10, 10, 10));
  cppLexer->setPaper (QColor (10, 10, 10));
  _lexer = cppLexer;
  _editor->setLexer (_lexer);

  // line numbers
  _editor->setMarginsForegroundColor (QColor (255, 255, 255));
  _editor->setMarginsBackgroundColor (QColor (20, 20, 20));
  _editor->setMarginType (2, QsciScintilla::TextMargin);

//    // code folding
//    _editor->setMarginType (3, QsciScintilla::SymbolMargin);
//    _editor->setFolding (QsciScintilla::BoxedTreeFoldStyle , 3);

  // highlight current line color
  _editor->setCaretLineBackgroundColor (QColor (0, 0, 0));

  // other features
  _editor->setBraceMatching (QsciScintilla::StrictBraceMatch);
  _editor->setAutoIndent (true);
  _editor->setIndentationWidth (2);
  _editor->setIndentationsUseTabs (false);

  _editor->setUtf8 (true);

  // auto completion
  _editor->autoCompleteFromAll ();
  _editor->setAutoCompletionSource (QsciScintilla::AcsAll);
  _editor->setAutoCompletionThreshold (1);

  _editor->setCaretLineVisible (true);
  _editor->setCaretForegroundColor (QColor (255, 255, 255));
  _editor->setMarginLineNumbers (2, true);
  _editor->setMarginsFont (font);
  QFontMetrics metrics (font);
  _editor->setMarginWidth (2, metrics.width ("9999"));
  _editor->
    setStyleSheet ("QListWidget { background-color: black; color: white; }");

  QVBoxLayout *l = new QVBoxLayout;
  l->addWidget (_menuBar);
  l->addWidget (_editor);
  l->setMargin (0);
  l->setSpacing (0);
  setLayout (l);
  
  _find_dialog = new find_dialog(_editor);

  connect (saveAction, SIGNAL (triggered ()), this, SLOT (saveFile ()));
  connect (searchAction, SIGNAL (triggered ()), this, SLOT (searchFile ()));
  connect (_editor, SIGNAL (modificationChanged (bool)),
	   this, SLOT (update_window_title (bool)));
}

SourceCodeEditor::~SourceCodeEditor ()
{
  delete ui;
}

void
SourceCodeEditor::setFileName (QString fileName)
{
  _currentFileName = fileName;
  QFile file (fileName);
  file.open (QFile::ReadOnly);
  if (file.isOpen ())
    {
      _editor->setText (QString::fromLocal8Bit (file.readAll ()));
      file.close ();
      _editor->setModified (false);
    }
  else
    {
    }
}

void
SourceCodeEditor::saveFile ()
{
  QFile file (_currentFileName);
  file.open (QFile::WriteOnly | QFile::Text);
  if (file.isOpen ())
    {
      file.write (_editor->text ().toLocal8Bit ());
      file.close ();
      _editor->setModified (false);
    }
  else
    {
    }
}

void
SourceCodeEditor::searchFile ()
{
  _find_dialog->show ();
}

void
SourceCodeEditor::update_window_title (bool modified)
{
  QFileInfo file_info (_currentFileName);
  setWindowTitle (file_info.fileName () + (modified ? "*" : ""));
}
